import { Args } from '@oclif/core';
import chalk from 'chalk';
import { ApifyCommand } from '../../lib/apify_command.js';
import { SharedRunOnCloudFlags, runActorOrTaskOnCloud } from '../../lib/commands/run-on-cloud.js';
import { simpleLog } from '../../lib/outputs.js';
import { getLocalUserInfo, getLoggedClientOrThrow } from '../../lib/utils.js';
export class TaskRunCommand extends ApifyCommand {
    static description = 'Runs a specific Actor remotely on the Apify cloud platform.\n' +
        'The Actor is run under your current Apify account. Therefore you need to be logged in by calling "apify login". ' +
        'It takes input for the Actor from the default local key-value store by default.';
    static flags = SharedRunOnCloudFlags('Task');
    static args = {
        taskId: Args.string({
            required: true,
            description: 'Name or ID of the Task to run (e.g. "my-task" or "E2jjCZBezvAZnX8Rb").',
        }),
    };
    async run() {
        const apifyClient = await getLoggedClientOrThrow();
        const userInfo = await getLocalUserInfo();
        const usernameOrId = userInfo.username || userInfo.id;
        const { id: taskId, userFriendlyId, title } = await this.resolveTaskId(apifyClient, usernameOrId);
        const runOpts = {
            waitForFinish: 2, // NOTE: We need to wait some time to Apify open stream and we can create connection
        };
        if (this.flags.build) {
            runOpts.build = this.flags.build;
        }
        if (this.flags.timeout) {
            runOpts.timeout = this.flags.timeout;
        }
        if (this.flags.memory) {
            runOpts.memory = this.flags.memory;
        }
        let url;
        let datasetUrl;
        const iterator = runActorOrTaskOnCloud(apifyClient, {
            actorOrTaskData: {
                id: taskId,
                userFriendlyId,
                title,
            },
            runOptions: runOpts,
            type: 'Task',
            printRunLogs: true,
        });
        for await (const yieldedRun of iterator) {
            url = `https://console.apify.com/actors/${yieldedRun.actId}/runs/${yieldedRun.id}`;
            datasetUrl = `https://console.apify.com/storage/datasets/${yieldedRun.defaultDatasetId}`;
        }
        simpleLog({
            message: [
                '',
                `${chalk.blue('Export results')}: ${datasetUrl}`,
                `${chalk.blue('View on Apify Console')}: ${url}`,
            ].join('\n'),
            stdout: true,
        });
    }
    async resolveTaskId(client, usernameOrId) {
        const { taskId } = this.args;
        // Full ID
        if (taskId?.includes('/')) {
            const task = await client.task(taskId).get();
            if (!task) {
                throw new Error(`Cannot find Task with ID '${taskId}' in your account.`);
            }
            return {
                id: task.id,
                userFriendlyId: `${usernameOrId}/${task.name}`,
                title: task.title,
                task,
            };
        }
        // Try fetching task directly by name
        if (taskId) {
            const task = await client.task(`${usernameOrId}/${taskId.toLowerCase()}`).get();
            if (!task) {
                throw new Error(`Cannot find Task with name '${taskId}' in your account.`);
            }
            return {
                id: task.id,
                userFriendlyId: `${usernameOrId}/${task.name}`,
                title: task.title,
                task,
            };
        }
        throw new Error('Please provide a valid Task ID or name.');
    }
}
//# sourceMappingURL=run.js.map