import process from 'node:process';
import { ACTOR_JOB_STATUSES } from '@apify/consts';
import { Flags } from '@oclif/core';
import chalk from 'chalk';
import { resolveInput } from './resolve-input.js';
import { CommandExitCodes } from '../consts.js';
import { error, run as runLog, success, warning } from '../outputs.js';
import { outputJobLog } from '../utils.js';
const TerminalStatuses = [
    ACTOR_JOB_STATUSES.SUCCEEDED,
    ACTOR_JOB_STATUSES.ABORTED,
    ACTOR_JOB_STATUSES.FAILED,
    ACTOR_JOB_STATUSES.TIMED_OUT,
];
export async function* runActorOrTaskOnCloud(apifyClient, options) {
    const cwd = process.cwd();
    const { actorOrTaskData, runOptions, type, waitForFinishMillis, inputOverride, silent, waitForRunToFinish, printRunLogs, } = options;
    const clientMethod = type === 'Actor' ? 'actor' : 'task';
    // Get input for actor
    const actorInput = resolveInput(cwd, inputOverride);
    if (!silent) {
        if (type === 'Actor') {
            runLog({
                message: `Calling ${type} ${actorOrTaskData.userFriendlyId} (${chalk.gray(actorOrTaskData.id)})\n`,
            });
        }
        else if (actorOrTaskData.title) {
            runLog({
                message: `Calling ${type} ${actorOrTaskData.title} (${actorOrTaskData.userFriendlyId}, ${chalk.gray(actorOrTaskData.id)})\n`,
            });
        }
        else {
            runLog({
                message: `Calling ${type} ${actorOrTaskData.userFriendlyId} (${chalk.gray(actorOrTaskData.id)})\n`,
            });
        }
    }
    let run;
    try {
        if (actorInput && type === 'Actor') {
            // TODO: For some reason we cannot pass json as buffer with right contentType into apify-client.
            // It will save malformed JSON which looks like buffer as INPUT.
            // We need to fix this in v1 during removing call under Actor namespace.
            run = await apifyClient[clientMethod](actorOrTaskData.id).start(actorInput.inputToUse, {
                ...runOptions,
                contentType: actorInput.contentType,
            });
        }
        else {
            run = await apifyClient[clientMethod](actorOrTaskData.id).start(undefined, runOptions);
        }
        // eslint-disable-next-line @typescript-eslint/no-explicit-any
    }
    catch (err) {
        // TODO: Better error message in apify-client-js
        if (err.type === 'record-not-found') {
            throw new Error(`${type} ${actorOrTaskData.userFriendlyId} (${actorOrTaskData.id}) not found!`);
        }
        throw err;
    }
    // Return the started run right away
    yield run;
    if (!silent && printRunLogs) {
        try {
            const res = await outputJobLog(run, waitForFinishMillis);
            if (res === 'timeouts') {
                console.error(`\n${chalk.gray('Timeout for printing logs was hit, there may be future logs.')}\n`);
            }
            else {
                console.error();
            }
        }
        catch (err) {
            warning({ message: 'Can not get log:' });
            console.error(err);
        }
    }
    run = (await apifyClient.run(run.id).get());
    if (waitForRunToFinish) {
        while (!TerminalStatuses.includes(run.status)) {
            run = (await apifyClient.run(run.id).get());
            if (TerminalStatuses.includes(run.status)) {
                break;
            }
            // Wait a second before checking again
            await new Promise((resolve) => setTimeout(resolve, 1000));
        }
    }
    if (!silent) {
        if (run.status === ACTOR_JOB_STATUSES.SUCCEEDED) {
            success({ message: `${type} finished.` });
        }
        else if (run.status === ACTOR_JOB_STATUSES.RUNNING) {
            warning({ message: `${type} is still running!` });
        }
        else if (run.status === ACTOR_JOB_STATUSES.ABORTED || run.status === ACTOR_JOB_STATUSES.ABORTING) {
            warning({ message: `${type} was aborted!` });
            process.exitCode = CommandExitCodes.RunAborted;
        }
        else {
            error({ message: `${type} failed!` });
            process.exitCode = CommandExitCodes.RunFailed;
        }
    }
    // Return the finished run
    yield run;
}
export const SharedRunOnCloudFlags = (type) => ({
    build: Flags.string({
        char: 'b',
        description: 'Tag or number of the build to run (e.g. "latest" or "1.2.34").',
        required: false,
    }),
    timeout: Flags.integer({
        char: 't',
        description: `Timeout for the ${type} run in seconds. Zero value means there is no timeout.`,
        required: false,
    }),
    memory: Flags.integer({
        char: 'm',
        description: `Amount of memory allocated for the ${type} run, in megabytes.`,
        required: false,
    }),
});
//# sourceMappingURL=run-on-cloud.js.map